/**
 * Скрипт закладок, имеет зависимости от scripts.js
 * @author N-Torrents.ru <feniwe@gmail.com>
 * @version 1.1
 */

var _nBookmarksReady = false; // общий флаг состояния элементов управления закладками (готовы ли они взаимодействовать с пользователем)
var _nBookmarksCacheTime = 60 * 60 * 24 * 90;
var _nBookmarksButtons;
var _nBookmarksDeleteButtons; // кнопки удаления материала из закладок в popup-окне
var _nBookmarksContent; // блок со списком закладок в popup-окне
var _nBookmarksPreloader; // прелоадер в popup-окне
var _nBookmarksError; // блок с текстом ошибки в popup-окне
var _nBookmarksPopupState = 'default'; // состояние popup-окна со списком закладок
var _nBookmarksHash = _nCookieGet('bookmarks_hash');
var _nBookmarksIds = _nCookieGet('bookmarks_ids');
var _nBookmarksState = { // объект с состоянием инициализации закзалок (создание хранилища, получение хеша), используется при загрузке страницы (позволяет не создавать хранилище хеша, если пользователь не пытался добавить ни одну закладку)
     oninit: 0x0, // хеша нет, как и хранилища хеща
    loading: 0x1, // выполняется запрос на создание хранилища
     active: 0x2, // закладки были активированы и можно добавлять/удалять/просматривать закладки
     failed: 0x3, // инициализация хранилища не удалась
    current: 0x0  // текущее состояние инициализации закладок
};

if (!_nBookmarksHash) {
    _nBookmarksFetchHash(false, _nBookmarksInit, _nBookmarksInit);
} else {
    _nBookmarksState.current = _nBookmarksState.active;
    _nBookmarksInit();
}

/**
 * Инициализиует элементы управления закладками, создает popup для их отображения
 * @return {undefined}
 */
function _nBookmarksInit() {
    // считываем идентификаторы материалов, находящихся в закладках
    var ids = _nCookieGet('bookmarks_ids');
    if (ids) {
        ids = '' + ids;
        _nBookmarksIds = ids.length > 0 ? ids.split('.') : [];
    }
    // инициализируем элементы для отображения закладок
    _nBookmarksInitPopup();
    // инициализируем кнопки добавления/удаления из закладок в материалах
    _nBookmarksInitControls();
    if (_nBookmarksState.current === _nBookmarksState.oninit) {
        _nBookmarksControlsEnabled(true);
    } else if (_nBookmarksButtons && _nBookmarksState.current !== _nBookmarksState.failed) {
        // синхронизация состояния кнопок добавить/убрать из закладок
        if (!_nBookmarksIds) {
            _nBookmarksGet(); // содержит вызов _nBookmarksControlsSync и _nBookmarksControlsEnabled
        } else {
            _nBookmarksControlsSync();
            _nBookmarksControlsEnabled(true);
        }
    }
}

/**
 * Создает на странице элементы для отображения закладок (пункты меню и само popup-окно)
 * @return {undefined}
 */
function _nBookmarksInitPopup() {
    var menus = document.querySelectorAll('.header-bar__profile-menu'), i;
    // добавляем кнопки открытия popup-окна
    if (menus) {
        for (i = 0; i < menus.length; i++) {
            var item = document.createElement('li');
            var link = document.createElement('a');
            link.innerText = 'Мои закладки';
            link.setAttribute('href', '#');
            link.className = 'popup-open popup-open_bookmarks';
            item.appendChild(link);
            menus[i].insertBefore(item, menus[i].children[menus[i].children.length - 1]);
        }
    }

    // добавляем popup-окно
    document.body.insertAdjacentHTML('beforeEnd', '<div class="popup popup_wide popup_bookmarks"><div class="popup__scroll-wrapper"><div class="popup__container"><button type="button" class="popup__close"></button><div class="popup__title">Закладки</div><div class="popup__content"><div class="preloader"></div><div class="popup__content-error"></div><div class="bookmarks-list"></div></div></div></div></div>');
    var popup = document.querySelector('.popup_bookmarks');
    _nBookmarksContent = popup.querySelector('.bookmarks-list');
    _nBookmarksPreloader = popup.querySelector('.preloader');
    _nBookmarksError = popup.querySelector('.popup__content-error');
    _nBookmarksContent.style.display = 'none';
    _nBookmarksError.style.display = 'none';
    _nBookmarksPreloader.style.display = 'block';
    _nPopupBehavior(popup);

    // добавляем функционал открытия popup-окон
    var triggers = document.querySelectorAll('.popup-open_bookmarks');
    if (triggers && triggers.length) {
        for (i = 0; i < triggers.length; i++) {
            triggers[i].addEventListener('click', function (event) {
                event = event || window.event;
                event.preventDefault();
                if (_nBookmarksState.current === _nBookmarksState.loading) {
                    console.error('Выполняется инициализация хранилища хеша, пожалуйста, подождите.');

                    return;
                } else if (_nBookmarksState.current === _nBookmarksState.failed) {
                    console.error('Не удалось инициализировать хранилище хеша. Функционал отключен.');

                    return;
                } else if (_nBookmarksState.current === _nBookmarksState.oninit) {
                    _nBookmarksState.current = _nBookmarksState.loading;
                    _nBookmarksStorage(function() {
                        _nBookmarksFetchHash(true, function() {
                            popup.classList.add('visible');
                            _nPopupOpened();
                        }, function() {
                            _nBookmarksState.current = _nBookmarksState.failed;
                        });
                    });
                } else if (!_nBookmarksReady) {
                    return;
                } else {
                    popup.classList.add('visible');
                    _nPopupOpened();
                }
            });
        }
    }
}

/**
 * Задает поведение кнопкам добавления/удаления из закладок в материалах сайта.
 * Вызов метода позволяет задать соответствующее поведение динамически добавленным на страницу материалам
 * @return {undefined}
 */
function _nBookmarksInitControls() {
    var actual = document.querySelectorAll('.entry__bookmark, .inner-entry__bookmark'), skip, i;
    if (!actual) {
        return;
    }
    if (_nBookmarksButtons && _nBookmarksButtons.length) {
        for (i = 0; i < actual.length; i++) {
            skip = false;
            for (var j = 0; j < _nBookmarksButtons.length; j++) {
                if (actual[i] === _nBookmarksButtons[j]) {
                    skip = true;
                    break;
                }
            }
            if (!skip) {
                behavior(actual[i]);
            }
        }
    } else {
        for (i = 0; i < actual.length; i++) {
            behavior(actual[i]);
        }
    }
    _nBookmarksButtons = actual;

    /**
     * Задает поведение конкретному элементу управления закладками
     * @param {HTMLElement} element - элемент управления (в материалах)
     * @return {undefined}
     */
    function behavior(element) {
        if (!element) {
            return;
        }
        element.addEventListener('click', function() {
            var id = _nBookmarksId(this.className);
            if (_nBookmarksState.current === _nBookmarksState.loading) {
                console.error('Выполняется инициализация хранилища хеша, пожалуйста, подождите.');

                return;
            } else if (_nBookmarksState.current === _nBookmarksState.failed) {
                console.error('Не удалось инициализировать хранилище хеша. Функционал отключен.');

                return;
            } else if (_nBookmarksState.current === _nBookmarksState.oninit) {
                _nBookmarksState.current = _nBookmarksState.loading;
                _nBookmarksStorage(function() {
                    _nBookmarksFetchHash(true, function() {
                        _nBookmarksGet(function() {
                            _nBookmarksAdd(id);
                        });
                    }, function() {
                        _nBookmarksState.current = _nBookmarksState.failed;
                    });
                });
            } else if (!_nBookmarksContains(_nBookmarksId(this.className))) {
                _nBookmarksAdd(id);
            } else {
                _nBookmarksDelete(id);
            }
        });
    }
}

/**
 * Синхронизирует состояние кнопок закладок в материалах
 * @return {undefined}
 */
function _nBookmarksControlsSync() {
    if (!_nBookmarksButtons) {
        return;
    }
    for (var i = 0; i < _nBookmarksButtons.length; i++) {
        var added_class = _nBookmarksAddedClass(_nBookmarksButtons[i].className);
        if (_nBookmarksContains(_nBookmarksId(_nBookmarksButtons[i].className))) {
            _nBookmarksButtons[i].classList.add(added_class);
            if (added_class === 'inner-entry__bookmark_added')
            {
                _nBookmarksButtons[i].innerText = 'В закладках';
            }
        } else {
            _nBookmarksButtons[i].classList.remove(added_class);
            if (added_class === 'inner-entry__bookmark_added')
            {
                _nBookmarksButtons[i].innerText = 'В закладки';
            }
        }
    }
}

/**
 * Активирует/деактивирует кнопки закладок (добавить/убрать из закладок)
 * @param {boolean} enabled - сделать ли элементы управления активными
 * @return {undefined}
 */
function _nBookmarksControlsEnabled(enabled) {
    if (enabled !== true) {
        _nBookmarksReady = false;
    }
    if (!_nBookmarksButtons) {
        return;
    }
    var i;
    for (i = 0; i < _nBookmarksButtons.length; i++) {
        if (enabled === true) {
            _nBookmarksButtons[i].removeAttribute('disabled');
        } else {
            _nBookmarksButtons[i].setAttribute('disabled', 'disabled');
        }
    }
    if (_nBookmarksDeleteButtons && _nBookmarksDeleteButtons.length > 0) {
        for (i = 0; i < _nBookmarksDeleteButtons.length; i++) {
            if (enabled === true) {
                _nBookmarksDeleteButtons[i].removeAttribute('disabled');
            } else {
                _nBookmarksDeleteButtons[i].setAttribute('disabled', 'disabled');
            }
        }
    }
    if (enabled === true) {
        _nBookmarksReady = true;
    }
}

/**
 * Выполняет запрос на получение хеша и, в случае необходимости, выполняет запрос на активацию закладок
 * @param {Boolean} retry - флаг, указывающий о повторном запросе после запроса на создание хранилища хеша
 * @param {Function} success_callback - функция вызова при успешном получении хеша
 * @param {Function} fail_callback - функция вызова при не успешном получении хеша (не считаем http-ошибки)
 * @return {undefined}
 */
function _nBookmarksFetchHash(retry, success_callback, fail_callback) {
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.ru/api/board/0-0-' + window._nUID + '-0-17', true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 8000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            console.error('Во время выполнения' + (retry ? ' повторного' : '') + ' запроса на получение хеша произошла ошибка.');
            return;
        }
        if (!this.responseText || this.responseText && this.responseText.indexOf('{"hash":"') === -1) {
            if (retry) {
                console.error('Хранилище хеша пусто.');
            }
            if (typeof fail_callback === 'function') {
                fail_callback(retry);
            }
            return;
        }
        var json_data = this.responseText.substring(this.responseText.indexOf('{"hash"'), this.responseText.indexOf('"}') + 2);
        var data = JSON.parse(json_data);
        _nCookieSet('bookmarks_hash', data.hash, {'expires' : _nBookmarksCacheTime});
        _nBookmarksHash = data.hash;
        _nBookmarksState.current = _nBookmarksState.active;
        if (typeof success_callback === 'function') {
            success_callback();
        }
    };
    xhr.ontimeout = function() {
        console.error('Превышено время выполнения' + (retry ? ' повторного' : '') + ' запроса на получение хеша.');
        _nBookmarksState.current = _nBookmarksState.oninit;
    };
    xhr.onerror = function() {
        console.error('Произошла ошибка в формировании запроса на получение хеша.');
        _nBookmarksState.current = _nBookmarksState.oninit;
    };
    xhr.send(null);
}

/**
 * Выполняет запрос на активацию закладок (инициализацию хранилища хеша)
 * @param {Function} callback - функция вызова при успешном/не успешном инициализации хранилища
 * @return {undefined}
 */
function _nBookmarksStorage(callback) {
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.com/v3/bookmarks.php?uid=' + window._nUID + '&hash=none&action=init', true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 10000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            console.error('Произошла HTTP ошибка при' + (retry ? ' повторном' : '') + ' запросе на создание хранилища хеша.');
            return;
        }
        var data = JSON.parse(this.responseText);
        if (!data || data && (data.error || !data.success)) {
            console.error((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');
        } else if (typeof callback === 'function') {
            callback();
        }
    };
    xhr.ontimeout = function() {
        console.error('Превышено время выполнения' + (retry ? ' повторного' : '') + ' запроса на создание хранилища хеша.');
        _nBookmarksState.current = _nBookmarksState.oninit;
    };
    xhr.onerror = function() {
        console.error('При формировании' + (retry ? ' повторного' : '') + ' запроса на создание хранилища хеша произошла ошибка.');
        _nBookmarksState.current = _nBookmarksState.oninit;
    };
    xhr.send(null);
}

/**
 * Выполняет запрос на получение списка идентификаторов закладок
 * @param {Function} callback - функция вызова при успешном/не успешном получении списка идентификаторов закладок
 * @return {undefined}
 */
function _nBookmarksGet(callback) {
    _nBookmarksControlsEnabled(false);
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.com/v3/bookmarks.php?uid=' + window._nUID + '&hash=' + (_nBookmarksHash ? _nBookmarksHash : 'none') + '&action=get', true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 10000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            _nBookmarksControlsEnabled(true);
            console.error('Произошла HTTP ошибка при получении списка идентификаторов закладок.');

            return;
        }
        var data = JSON.parse(this.responseText);
        if (!data || data && (data.error || !data.success)) {
            _nBookmarksControlsEnabled(true);
            console.error((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');

            return;
        } else {
            _nBookmarksIds = data.success.bookmarks;
        }
        _nCookieSet('bookmarks_ids', _nBookmarksIds.join('.'), {'expires' : _nBookmarksCacheTime});
        _nBookmarksControlsSync();
        _nBookmarksControlsEnabled(true);
        if (typeof callback === 'function') {
            callback();
        }
    };
    xhr.ontimeout = function() {
        _nBookmarksControlsEnabled(true);
        console.error('Превышено время выполнения запроса на получение списка идентификаторов закладок.');
    };
    xhr.onerror = function() {
        _nBookmarksControlsEnabled(true);
        console.error('При формировании запроса на получение списка идентификаторов закладок произошла ошибка.');
    };
    xhr.send(null);
}

/**
 * Выполняет запрос на получение списка закладок
 * @return {undefined}
 */
function _nBookmarksList() {
    _nBookmarksControlsEnabled(false);
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.com/v3/bookmarks.php?uid=' + window._nUID + '&hash=' + (_nBookmarksHash ? _nBookmarksHash : 'none') + '&action=list', true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 10000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            _nBookmarksControlsEnabled(true);
            _nBookmarksPopupError('Произошла HTTP ошибка при получении списка закладок.');
            console.error('Произошла HTTP ошибка при получении списка закладок.');

            return;
        }
        var data = JSON.parse(this.responseText);
        if (!data || data && (data.error || !data.success)) {
            _nBookmarksPopupState = 'error';
            _nBookmarksControlsEnabled(true);
            _nBookmarksPopupError((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');
            console.error((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');

            return;
        }
        if (!data.success || !data.success.bookmarks) {
            _nBookmarksPopupState = 'error';
            _nBookmarksControlsEnabled(true);
            _nBookmarksPopupError('Не удалось разобрать ответ сервера на запрос списка закладок.');
            console.error('Не удалось разобрать ответ сервера на запрос списка закладок.');

            return;
        }
        _nBookmarksIds = data.success.bookmarks;
        _nCookieSet('bookmarks_ids', _nBookmarksIds.join('.'), {'expires' : _nBookmarksCacheTime});
        _nBookmarksControlsSync();
        _nBookmarksPopupState = 'finished';
        if (!data.success.list || data.success.list && data.success.list.length === 0) {
            _nBookmarksPopupError('Вы не добавили ни одной закладки');
        } else {
            _nBookmarksRender(data.success.list);
            _nBookmarksPopupReady();
        }
        _nBookmarksControlsEnabled(true);
    };
    xhr.ontimeout = function() {
        _nBookmarksPopupState = 'error';
        _nBookmarksControlsEnabled(true);
        _nBookmarksPopupError('Превышено время выполнения запроса на получение списка закладок.');
        console.error('Превышено время выполнения запроса на получение списка закладок.');
    };
    xhr.onerror = function() {
        _nBookmarksPopupState = 'error';
        _nBookmarksControlsEnabled(true);
        _nBookmarksPopupError('При формировании запроса на получение списка закладок произошла ошибка.');
        console.error('При формировании запроса на получение списка закладок произошла ошибка.');
    };
    xhr.send(null);
}

/**
 * Выполняет запрос на добавление закладки
 * @param {Number} id - идентификатор материала
 * @return {undefined}
 */
function _nBookmarksAdd(id) {
    _nBookmarksControlsEnabled(false);
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.com/v3/bookmarks.php?uid=' + window._nUID + '&hash=' + (_nBookmarksHash ? _nBookmarksHash : 'none') + '&action=add&ids=' + id, true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 10000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            _nBookmarksControlsEnabled(true);
            console.error('Произошла HTTP ошибка при добавлении закладки.');

            return;
        }
        var data = JSON.parse(this.responseText);
        if (!data || data && (data.error || !data.success)) {
            _nBookmarksControlsEnabled(true);
            console.error((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');

            return;
        } else {
            _nBookmarksIds = data.success.bookmarks;
        }
        _nCookieSet('bookmarks_ids', _nBookmarksIds.join('.'), {'expires' : _nBookmarksCacheTime});
        _nBookmarksControlsSync();
        _nBookmarksPopupState = 'default'; // позволяем при следующем открытии окна начать подгрузку свежего списка закладок
        _nBookmarksControlsEnabled(true);
    };
    xhr.ontimeout = function() {
        _nBookmarksControlsEnabled(true);
        console.error('Превышено время выполнения запроса на добавление закладки.');
    };
    xhr.onerror = function() {
        _nBookmarksControlsEnabled(true);
        console.error('При формировании запроса на добавление закладки произошла ошибка.');
    };
    xhr.send(null);
}

/**
 * Выполняет запрос на добавление закладки
 * @param {Number} id - идентификатор материала
 * @return {undefined}
 */
function _nBookmarksDelete(id) {
    _nBookmarksControlsEnabled(false);
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', 'http://n-torrents.com/v3/bookmarks.php?uid=' + window._nUID + '&hash=' + (_nBookmarksHash ? _nBookmarksHash : 'none') + '&action=delete&ids=' + id, true);
    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    xhr.timeout = 10000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            _nBookmarksControlsEnabled(true);
            console.error('Произошла HTTP ошибка при удалении закладки.');

            return;
        }
        var data = JSON.parse(this.responseText);
        if (!data || data && (data.error || !data.success)) {
            _nBookmarksControlsEnabled(true);
            console.error((data && data.error) ? data.error.message : 'В ответе сервера отсуствуют данные.');

            return;
        } else {
            _nBookmarksIds = data.success.bookmarks;
        }
        _nCookieSet('bookmarks_ids', _nBookmarksIds.join('.'), {'expires' : _nBookmarksCacheTime});
        _nBookmarksControlsSync();
        if (data.success.deleted && data.success.deleted.length > 0) {
            _nBookmarksRemoveRendered(data.success.deleted);
        }
        _nBookmarksControlsEnabled(true);
    };
    xhr.ontimeout = function() {
        _nBookmarksControlsEnabled(true);
        console.error('Превышено время выполнения запроса на удаление закладки.');
    };
    xhr.onerror = function() {
        _nBookmarksControlsEnabled(true);
        console.error('При формировании запроса на удаление закладки произошла ошибка.');
    };
    xhr.send(null);
}

/*
 * Получает идентификатор материала
 * @param {String} className - значение аттрибута class у кнопки
 * @return {Number}
 */
function _nBookmarksId(className) {
    if (typeof className !== 'string') {
        return 0;
    }
    var matches = className.match(/\d+/);
    if (!matches || matches && matches.length === 0) {
        return 0;
    }

    return matches[0];
}

/**
 * Проверяет находится ли материал в списке закладок
 * @param {Number} id - идентификатор материала
 * @return {Boolean}
 */
function _nBookmarksContains(id) {
    if (!id) {
        return false;
    }
    if (!_nBookmarksIds || _nBookmarksIds && _nBookmarksIds.length === 0) {
        return false;
    }
    id = +id;
    for (var i = 0; i < _nBookmarksIds.length; i++) {
        if (+_nBookmarksIds[i] === id) {
            return true;
        }
    }

    return false;
}

/*
 * Получает название класса, который отвечает за состояние кнопки, когда материал в закладках
 * @param {String} className - значение аттрибута class у кнопки
 * @return {String}
 */
function _nBookmarksAddedClass(className) {
    if (typeof className !== 'string') {
        return 'added';
    }
    var classes = className.split(' ');
    for (var i = 0; i < classes.length; i++) {
        switch(classes[i]) {
            case 'entry__bookmark':
                return 'entry__bookmark_added';
            case 'inner-entry__bookmark':
                return 'inner-entry__bookmark_added';
        }
    }

    return 'added';
}

/**
 * Отрисовывает список закладок
 * @param {Object[]} rows - массив элементов
 * @return {undefined}
 */
function _nBookmarksRender(rows) {
    var html = '', i, currupted_rows = 0;
    for (i = 0; i < rows.length; i++) {
        if (!rows[i]) {
            currupted_rows++;
        }
    }
    if (currupted_rows === rows.length) {
        html = 'Ошибка генерации закладок. Возможно, обработчик закладок не смог получить данные о материале.';
    } else {
        for (i = 0; i < rows.length; i++) {
            html += '<div class="bookmark-entry bookmark-entry_id-' + rows[i].id + '">';
            html += '<a href="' + rows[i].entry_url + '" class="aspect-ratio aspect-ratio_image" style="background-image: url(\'' + rows[i].screenshot + '\');"><img src="' + rows[i].screenshot + '" alt="' + rows[i].title + '" title="' + rows[i].title + '"></a>';
            html += '<a href="' + rows[i].entry_url + '" class="bookmark-entry__link" rel="nofollow">' + rows[i].title_short + '</a>';
            html += '<button type="button" class="bookmark-entry__delete bookmark-entry__delete-' + rows[i].id + '" title="Удалить из закладок"></button>';
            html += '</div>';
        }
    }
    _nBookmarksContent.innerHTML = html;
    _nBookmarksDeleteButtons = _nBookmarksContent.querySelectorAll('.bookmark-entry__delete');
    if (_nBookmarksDeleteButtons && _nBookmarksDeleteButtons.length > 0) {
        for (i = 0; i < _nBookmarksDeleteButtons.length; i++) {
            _nBookmarksDeleteButtons[i].addEventListener('click', _nBookmarksDeleteButtonClick);
        }
    }
}

/*
 * Запускает действия, которые должны быть выполнены именно при открытии popup-окна
 * @return {undefined}
 */
function _nPopupOpened() {
    switch(_nBookmarksPopupState) {
        case 'default':
        case 'error': // состояние, когда при открытии окна ничто не мешает начать подгрузку закладок
            _nBookmarksPopupState = 'loading';
            _nBookmarksPopupLoading();
            _nBookmarksList();
            break;
        case 'loading': // состояние, когда идёт загрузка списка закладок
            break;
        case 'finished': // состояние когда закладки отрисованы
            break;
    }
}

/*
 * Отображает в popup-окне прелоадер
 * @return {undefined}
 */
function _nBookmarksPopupLoading() {
    _nBookmarksContent.style.display = 'none';
    _nBookmarksError.style.display = 'none';
    _nBookmarksPreloader.style.display = 'block';
}

/*
 * Отображает в popup-окне текст ошибки
 * @param {String} error - текст ошибки
 * @return {undefined}
 */
function _nBookmarksPopupError(error) {
    _nBookmarksError.innerText = error;
    _nBookmarksContent.style.display = 'none';
    _nBookmarksError.style.display = 'block';
    _nBookmarksPreloader.style.display = 'none';
}

/*
 * Отображает в popup-окне блок с содержимым
 * @return {undefined}
 */
function _nBookmarksPopupReady() {
    _nBookmarksContent.style.display = 'block';
    _nBookmarksError.style.display = 'none';
    _nBookmarksPreloader.style.display = 'none';
}

/*
 * Клик по кнопке удаления закладки
 * @return {undefined}
 */
function _nBookmarksDeleteButtonClick() {
    if (!_nBookmarksReady) {
        return;
    }
    _nBookmarksDelete(_nBookmarksId(this.className));
}

/*
 * Удаляет из popup-окна блоки закладок с указанными идентификаторами
 * @param {Number[]} ids - список идентификаторов материалов, удаленных из закладок
 * @return {undefined}
 */
function _nBookmarksRemoveRendered(ids) {
    var cssQuery = [], i;
    for (i = 0; i < ids.length; i++) {
        cssQuery.push('.bookmark-entry_id-' + ids[i]);
    }
    if (cssQuery.length === 0) {
        return;
    }
    var remove = _nBookmarksContent.querySelectorAll(cssQuery.join(', '));
    if (remove && remove.length) {
        for (i = 0; i < remove.length; i++) {
            _nBookmarksContent.removeChild(remove[i]);
        }
    }
    if (_nBookmarksContent.children.length === 0) {
        _nBookmarksContent.innerText = 'Вы не добавили ни одной закладки';
    }
}