/**
 * Скрипт мини-чата. Возможна подгрузка файла с аттрибутом async или defer
 * @author N-Torrents.ru <feniwe@gmail.com>
 * @version 2.0
 */

var _nChatList;
var _nChatInput;
var _nChatSubmit;
var _nChatReady = true;
var _nChatMinLength = 3;
var _nChatInterval;
var _nChatUpdateTimeout = 8000;
var _nChatCurrentHtml;

_nChatPolyfills();
_nChatInit();

/**
 * Инициализирует чат
 * @return {Void}
 */
function _nChatInit() {
    // инициализация элементов
    _nChatList = document.querySelector('.chat__list-wrapper');
    _nChatInput = document.querySelector('.chat__form-input input[type="text"]');
    _nChatSubmit = document.querySelector('.chat__form-submit');
    _nChatEmoji = document.querySelector('.chat__form-emoji');
    if (!_nChatList || !_nChatInput || !_nChatSubmit) {
        return;
    }
    // поведение чата
    _nChatUpdate();
    _nChatInterval = setInterval(_nChatUpdate, _nChatUpdateTimeout);
    if (window._nUID) {
        _nChatSubmit.addEventListener('click', function() {
            var message = _nChatInput.value.trim();
            if (!message || message && message.length < _nChatMinLength) {
                return;
            }
            _nChatPost(message);
        });
    }
}

/**
 * Выполняет обновление списка сообщение
 * @return {Void}
 */
function _nChatUpdate() {
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('GET', '/mchat/', true);
    xhr.timeout = 8000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            return;
        }
        _nChatResult(this.responseText);
    };
    xhr.send();
}

/**
 * Отправляет сообщение
 * @param {String} message - текст сообщения
 * @return {Void}
 */
function _nChatPost(message) {
    if (!_nChatReady) {
        return;
    }
    _nChatReady = false;
    _nChatEnabled(false);
    var params = {
        'mcmessage': message,
        'a': 18,
        'ajax': 1,
        'numa': 0,
        '_tp_': 'xml'
    };
    var boundary = String(Math.random()).slice(2);
    var boundaryMiddle = '--' + boundary + '\r\n';
    var boundaryLast = '--' + boundary + '--\r\n';
    var body = ['\r\n'];
    for (var key in params) {
        body.push('Content-Disposition: form-data; name="' + key + '"\r\n\r\n' + params[key] + '\r\n');
    }
    body = body.join(boundaryMiddle) + boundaryLast;
    var XHR = ("onload" in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
    var xhr = new XHR();
    xhr.open('POST', '/mchat/?' + String(Math.random()).slice(2) + '.' + String(Math.random()).slice(2), true);
    xhr.setRequestHeader('Content-Type', 'multipart/form-data; boundary=' + boundary);
    xhr.timeout = 8000;
    xhr.onload = function() {
        if (this.readyState !== 4) {
            return;
        }
        if (this.status !== 200) {
            _nChatReady = true;
            _nChatEnabled(true);

            return;
        }
        _nChatInput.value = '';
        _nChatReady = true;
        _nChatEnabled(true);
    };
    xhr.ontimeout = function() {
        _nChatReady = true;
        _nChatEnabled(true);
    };
    xhr.send(body);
}

/**
 * Делает элементы управления активными/не активными
 * @return {Void}
 */
function _nChatEnabled(enabled) {
    if (enabled === true) {
        _nChatInput.removeAttribute('disabled');
        _nChatSubmit.removeAttribute('disabled');
        if (_nChatEmoji) {
            _nChatEmoji.removeAttribute('disabled');
        }
    } else {
        _nChatInput.setAttribute('disabled', 'disabled');
        _nChatSubmit.setAttribute('disabled', 'disabled');
        if (_nChatEmoji) {
            _nChatEmoji.setAttribute('disabled', 'disabled');
        }
    }
}

/**
 * Обновляет список сообщений из полученного html-кода ответа сервера
 * @return {Void}
 */
function _nChatResult(html) {
    if (!html || html && html.indexOf('<article') == -1 || _nChatCurrentHtml == html) {
        return;
    }
    var content = html.substring(html.indexOf('<article'), html.lastIndexOf('</article>') + '</article>'.length);
    if (!content) {
        content = 'Нет сообщений';
    } else {
        content = content.replace(/<(\/?)article((?:\s+class="[\w\d-_]+")?)>/g, '<$1div$2>');
    }
    _nChatList.innerHTML = content;
    var authors = document.querySelectorAll('.chat__message-author a');
    if (!authors || authors && authors.length == 0) {
        return;
    }
    for (var i = 0; i < authors.length; i++) {
        authors[i].addEventListener('click', function(event) {
            event = event || window.event;
            event.preventDefault();
            _nChatInput.focus();
            var value = _nChatInput.value ? _nChatInput.value : '';
            var start = ('selectionStart' in _nChatInput) ? _nChatInput.selectionStart : value.length;
            var end = ('selectionEnd' in _nChatInput) ? _nChatInput.selectionEnd : value.length;
            if (start > end) {
                var tmp = start;
                start = end;
                end = tmp;
            }
            if (!value.length || start == 0 && end == value.length) {
                // если в поле нет текста или выделен сразу весь текст, то заменяем значение полностью
                _nChatInput.value = '[b]' + this.innerText + '[/b], ';
            } else {
                _nChatInput.value = (start > 0 ? value.substring(0, start) : '') + '[b]' + this.innerText + '[/b], ' + (end < value.length ? value.substring(end) : '');
            }
        });
    }
}

/**
 * Полифиллы
 */
function _nChatPolyfills() {
    if ( (!document.addEventListener) && document.attachEvent ) {
        Object.defineProperty(Element.prototype, 'addEventListener', function(eventName, func) {
            var self = this;
            self['e' + eventName + func] = fn;
            self[eventName + func] = function() {
                obj['e' + eventName + func]( window.event );
            }
            self.attachEvent('on' + eventName, self[eventName + func] );
        });
    }
    if (!String.prototype.trim) {
        String.prototype.trim = function() {
            return this.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
        };
    }
}